from telegram.ext import Updater, CommandHandler

from .commands.help import help
from .commands.quote import quote


def run(config) -> None:
    updater = Updater(config.TELEGRAM_TOKEN)
    dispatcher = updater.dispatcher

    dispatcher.add_handler(CommandHandler("help", help))
    dispatcher.add_handler(CommandHandler("quote", quote))

    updater.start_polling()
    updater.idle()



import logging

from poetrybot.config import Config
from poetrybot.database import store
from poetrybot.telegram.bot import run

logging.basicConfig(
    level=logging.INFO,
    format="%(name)s - %(levelname)s - %(message)s",
)

config = Config.from_environ()
logger = logging.getLogger(__name__)


def main():
    logger.info("Starting poetrybot")

    store.connect(config.DATABASE_URL)

    run(config)


if __name__ == "__main__":
    main()



from marshmallow import Schema, fields


class PoemSchema(Schema):
    id = fields.Int(dump_only=True)
    title = fields.String()
    verses = fields.String(required=True)
    poet_id = fields.Int(required=True)



from flask import jsonify
from werkzeug.http import HTTP_STATUS_CODES


def error(status_code, message=None):
    """Return the error specified by the status_code.

    If message is present, attach also it to the response.
    """
    payload = {"error": HTTP_STATUS_CODES.get(status_code, "Unknown")}
    if message:
        payload["message"] = message

    response = jsonify(payload)
    response.status_code = status_code

    return response



from flask import jsonify, request
from marshmallow import ValidationError

from poetrybot.database import store
from poetrybot.database.models import Poet

from ..errors import error

from . import bp
from .schemas import PoetSchema

poet_schema = PoetSchema()
poets_schema = PoetSchema(many=True)


@bp.route("", methods=["GET"])
def get_poets():

    with store.get_session() as s:
        poets = s.query(Poet).all()

    return jsonify(poets_schema.dump(poets))


@bp.route("", methods=["POST"])
def create_poet():
    data = request.get_json() or {}

    try:
        data = poet_schema.load(data)
    except ValidationError as err:
        return error(400, err.messages)

    created = None
    with store.get_session() as s:

        if s.query(Poet).filter(Poet.name == data["name"]).first():
            return error(400, "this poet is already present")

        poet = Poet(name=data["name"])
        s.add(poet)
        s.commit()

        created = poet_schema.dump(poet)

    response = jsonify(created)
    response.status_code = 201
    return response


@bp.route("/<int:id>", methods=["GET"])
def get_poet(id):

    with store.get_session() as s:
        poet = s.query(Poet).filter(Poet.id == id).first()

    if not poet:
        return error(404)

    return jsonify(poet_schema.dump(poet))


@bp.route("/<int:id>", methods=["PUT"])
def update_poet(id):
    data = request.get_json() or {}

    try:
        data = poet_schema.load(data)
    except ValidationError as err:
        return error(400, err.messages)

    updated = None
    with store.get_session() as s:
        poet = s.query(Poet).filter(Poet.id == id).first()

        if not poet:
            return error(404)

        poet.name = data["name"]
        s.commit()

        updated = poet_schema.dump(poet)

    return jsonify(updated)


@bp.route("/<int:id>", methods=["DELETE"])
def delete_poet(id):

    with store.get_session() as s:
        poet = s.query(Poet).filter(Poet.id == id).first()

        if not poet:
            return error(404)

        s.delete(poet)
        s.commit()

    return "", 204



from flask import jsonify, request
from marshmallow import ValidationError

from poetrybot.database import store
from poetrybot.database.models import Poem

from ..errors import error

from . import bp
from .schemas import PoemSchema

poem_schema = PoemSchema()
poems_schema = PoemSchema(many=True)


@bp.route("", methods=["GET"])
def get_poems():

    with store.get_session() as s:
        poems = s.query(Poem).all()

    return jsonify(poems_schema.dump(poems))


@bp.route("", methods=["POST"])
def create_poem():
    data = request.get_json(silent=True) or {}

    try:
        data = poem_schema.load(data)
    except ValidationError as err:
        return error(400, err.messages)

    created = None
    with store.get_session() as s:

        poem = Poem(**data)
        s.add(poem)
        s.commit()

        created = poem_schema.dump(poem)

    response = jsonify(created)
    response.status_code = 201
    return response


@bp.route("/<int:id>", methods=["GET"])
def get_poem(id):

    with store.get_session() as s:
        poem = s.query(Poem).filter(Poem.id == id).first()

    if not poem:
        return error(404)

    return jsonify(poem_schema.dump(poem))


@bp.route("/<int:id>", methods=["PUT"])
def update_poem(id):
    data = request.get_json(silent=True) or {}

    try:
        data = poem_schema.load(data)
    except ValidationError as err:
        return error(400, err.messages)

    updated = None
    with store.get_session() as s:
        poem = s.query(Poem).filter(Poem.id == id).first()

        if not poem:
            return error(404)

        for key in data:
            setattr(poem, key, data[key])
        s.commit()

        updated = poem_schema.dump(poem)

    return jsonify(updated)


@bp.route("/<int:id>", methods=["DELETE"])
def delete_poem(id):

    with store.get_session() as s:
        poem = s.query(Poem).filter(Poem.id == id).first()

        if not poem:
            return error(404)

        s.delete(poem)
        s.commit()

    return "", 204



from contextlib import contextmanager

from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker, scoped_session

Base = declarative_base()


class Store:
    """A database access layer for poetrybot."""

    def __init__(self) -> None:
        self.connection_string = None
        self.engine = None

    def connect(self, connection_string: str) -> None:
        self.connection_string = connection_string
        self.engine = create_engine(self.connection_string)
        Base.metadata.create_all(self.engine)
        self.session = scoped_session(
            sessionmaker(autocommit=False, autoflush=False, bind=self.engine)
        )

    @contextmanager
    def get_session(self):
        try:
            yield self.session()
        finally:
            self.session.remove()

    def rollback(self):
        self.session.rollback()


store = Store()



from flask import Flask, request

from poetrybot.config import Config
from poetrybot.database import store

from .poems import bp as poems_bp
from .poets import bp as poets_bp
from .users import bp as users_bp


def create_app(config=None):
    """Create a Flask app with specified configuration.

    If config is None the configuration is made using environment variables.
    """
    if config is None:
        config = Config.from_environ()
    store.connect(config.DATABASE_URL)

    app = Flask(__name__)

    @app.before_request
    def check_auth_token():
        """Check every request for the Authorization header."""
        auth_token = request.headers.get("Authorization")

        if auth_token != config.AUTH_TOKEN:
            return "", 401

    app.register_blueprint(poems_bp, url_prefix="/poems")
    app.register_blueprint(poets_bp, url_prefix="/poets")
    app.register_blueprint(users_bp, url_prefix="/users")

    return app



from flask import jsonify, request
from marshmallow import ValidationError

from poetrybot.database import store
from poetrybot.database.models import User

from ..errors import error

from . import bp
from .schemas import UserSchema, UserEditSchema

user_edit_schema = UserEditSchema()
user_schema = UserSchema()
users_schema = UserSchema(many=True)


@bp.route("", methods=["GET"])
def get_users():

    with store.get_session() as s:
        users = s.query(User).all()

    return jsonify(users_schema.dump(users))


@bp.route("", methods=["POST"])
def create_user():
    data = request.get_json(silent=True) or {}

    try:
        data = user_schema.load(data)
    except ValidationError as err:
        return error(400, err.messages)

    created = None
    with store.get_session() as s:
        if s.query(User).filter(User.id == data["id"]).first():
            return error(400, "user with the specified id already exists")

        user = User(**data)
        s.add(user)
        s.commit()

        created = user_schema.dump(user)

    response = jsonify(created)
    response.status_code = 201
    return response


@bp.route("/<int:id>", methods=["GET"])
def get_user(id):

    with store.get_session() as s:
        user = s.query(User).filter(User.id == id).first()

    if not user:
        return error(404)

    return jsonify(user_schema.dump(user))


@bp.route("/<int:id>", methods=["PUT"])
def update_user(id):
    data = request.get_json() or {}

    try:
        data = user_edit_schema.load(data)
    except ValidationError as err:
        return error(400, err.messages)

    updated = None
    with store.get_session() as s:
        user = s.query(User).filter(User.id == id).first()

        if not user:
            return error(404)

        user.name = data["name"]
        s.commit()

        updated = user_schema.dump(user)

    return jsonify(updated)


@bp.route("/<int:id>", methods=["DELETE"])
def delete_user(id):

    with store.get_session() as s:
        user = s.query(User).filter(User.id == id).first()

        if not user:
            return error(404)

        s.delete(user)
        s.commit()

    return "", 204



"""Configuration module for poetrybot."""

import environ


@environ.config(prefix="")
class Config:
    AUTH_TOKEN = environ.var()
    DATABASE_URL = environ.var()
    TELEGRAM_TOKEN = environ.var()



from marshmallow import Schema, fields, ValidationError


def validate_id(n):
    """Validate the id field."""
    # https://core.telegram.org/bots/api#chat
    # We use an heuristic approach here, it seems that at least
    if n < 100000:
        raise ValidationError("id must be greater than 99999.")


class UserSchema(Schema):
    """Schema for users.

    Used everywhere except during users' edit.
    """

    id = fields.Int(validate=validate_id, required=True)
    name = fields.String(required=True)


class UserEditSchema(Schema):
    """Schema for users edit."""

    name = fields.String(required=True)



"""Database models for poetrybot."""
from sqlalchemy import Column, Integer, String, Text, ForeignKey
from sqlalchemy.orm import relationship, backref


from . import Base


class User(Base):
    """An user of poetrybot."""

    __tablename__ = "users"

    id = Column(Integer, primary_key=True, index=True)
    name = Column(String(length=150), nullable=False)

    def __repr__(self):
        return f"User(id={self.id}, name={self.name})"


class Poet(Base):
    """A poet."""

    __tablename__ = "poets"

    id = Column(Integer, primary_key=True, index=True)
    name = Column(String(length=150, collation="NOCASE"), nullable=False, unique=True)

    def __repr__(self):
        return f"Poet(id={self.id}, name={self.name})"


class Poem(Base):
    """A poem."""

    __tablename__ = "poems"

    id = Column(Integer, primary_key=True, index=True)
    title = Column(String(length=255, collation="NOCASE"), nullable=True)
    verses = Column(Text(collation="NOCASE"), nullable=False)
    poet_id = Column(Integer, ForeignKey("poets.id"), nullable=False)

    author = relationship("Poet", backref=backref("poems"))



import logging
import re

from telegram import Update, ParseMode
from telegram.ext import CallbackContext

from poetrybot.database import store
from poetrybot.database.api import get_a_random_poem, is_user_in_allow_list

logger = logging.getLogger(__name__)

QUOTE_REGEX_AUTHOR = re.compile(
    r"""
(?:/quote)                      # /quote
(?:@\w+)?                       # name of the bot
\s+                             # one or more spaces
(?P<author>[A-Za-z\s]+)         # the author
""",
    re.VERBOSE,
)

QUOTE_REGEX_AUTHOR_ABOUT = re.compile(
    r"""
(?:/quote)                      # /quote
(?:@\w+)?                       # name of the bot
\s+                             # one or more spaces
(?P<author>[A-Za-z\s]+)?        # the author (is optional)
(?:about)                       # about
(?P<argument>[A-Za-z\s]+)?      # the argument (is optional)
""",
    re.VERBOSE,
)


def quote(update: Update, context: CallbackContext) -> None:
    """Get a poem."""
    with store.get_session() as s:

        user_id = update.effective_user.id
        if not is_user_in_allow_list(s, user_id=user_id):
            logger.warning(
                "Telegram user with id '{}' and username"
                " '{}' tried to get a quote.".format(
                    user_id, update.effective_user.username
                )
            )
            return

        author, argument = parse_quote(update.message.text)
        poem = get_a_random_poem(s, author=author, argument=argument)

        reply = f"{poem.verses}\n\n_{poem.author.name}_" if poem else "No quote found!"

    context.bot.send_message(
        chat_id=update.effective_chat.id, text=reply, parse_mode=ParseMode.MARKDOWN
    )


def parse_quote(text: str):
    """Parse text to extract author and argument."""
    author = None
    argument = None

    matched = QUOTE_REGEX_AUTHOR_ABOUT.match(text)
    if not matched:
        matched = QUOTE_REGEX_AUTHOR.match(text)

    if matched:
        try:
            author = matched.group("author").strip()
        except (IndexError, AttributeError):
            pass

        try:
            argument = matched.group("argument").strip()
        except (IndexError, AttributeError):
            pass

    return author, argument



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['poetrybot',
 'poetrybot.database',
 'poetrybot.telegram',
 'poetrybot.telegram.commands',
 'poetrybot.web',
 'poetrybot.web.poems',
 'poetrybot.web.poets',
 'poetrybot.web.users']

package_data = \
{'': ['*']}

install_requires = \
['Flask>=1.1.2,<2.0.0',
 'SQLAlchemy>=1.4.3,<2.0.0',
 'environ-config>=20.1.0,<21.0.0',
 'marshmallow>=3.11.1,<4.0.0',
 'python-telegram-bot>=13.4,<14.0']

entry_points = \
{'console_scripts': ['poetrybot = poetrybot.main:main']}

setup_kwargs = {
    'name': 'poetrybot',
    'version': '0.1.0',
    'description': 'poetrybot is a Telegram bot to quote poems between friends.',
    'long_description': None,
    'author': 'Daniele Tricoli',
    'author_email': 'eriol@mornie.org',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from telegram import Update
from telegram.ext import CallbackContext


def help(update: Update, context: CallbackContext) -> None:
    """Give help to users."""
    update.message.reply_text("Hello! Use /quote to get a random poem!")



from sqlalchemy.orm import contains_eager
from sqlalchemy.orm.exc import NoResultFound
from sqlalchemy.sql.expression import func

from .models import Poem, User, Poet


def get_a_random_poem(session, author=None, argument=None):
    """Return a random poem.

    Filter by author and argument if specified.
    """
    try:
        query = session.query(Poem).join(Poem.author)

        if author is not None:
            query = query.options(contains_eager(Poem.author)).filter(
                Poet.name.like(f"%{author}%")
            )
        if argument is not None:
            query = query.filter(Poem.verses.like(f"%{argument}%"))

        return query.order_by(func.random()).limit(1).one()

    except NoResultFound:
        return None


def is_user_in_allow_list(session, user_id):
    """Return True if the user is inside the allow list."""
    return True if session.query(User).filter(User.id == user_id).count() else False
